/***********************************************************************
*  Warsaw University
*  Michal Pietrusinski 2001 (mpietrus@fuw.edu.pl>)
*  some small changing by Karol Bunkowski (kbunkow@fuw.edu.pl) 	
*  
*  Base class for VME classe.
*  
/***********************************************************************/
#ifndef TVMEINTERFACE_HH
#define TVMEINTERFACE_HH

#include <stdexcept>
#include <string>

//VME exceptions:
class EVME : public std::runtime_error
{
public:
    EVME( std::string msg = "" )
	: runtime_error( "VME Error:" + msg ) {}
    virtual ~EVME() {}
};


class EVMEOpen : public EVME {
public:
    EVMEOpen( std::string msg = "" )
	: EVME(": could not open " + msg) {}
     virtual ~EVMEOpen() {}
};

class EVMEHardware : public EVME {
public:
    EVMEHardware( std::string msg = "" )
	: EVME(": hardware error " + msg) {}
     virtual ~EVMEHardware() {}
};

class EVMEWrite : public EVME {
public:
    EVMEWrite( std::string msg = "" )
	: EVME(": write error " + msg) {}
     virtual ~EVMEWrite() {}
};

class EVMERead : public EVME {
public:
    EVMERead( std::string msg = "" )
	: EVME(": read error " + msg) {}
     virtual ~EVMERead() {}
};


class TVMEInterface {

public:
    TVMEInterface() {}
    virtual ~TVMEInterface() {}

	virtual void Reset() = 0;

    virtual void Write (unsigned long int address, char * data, size_t  count)  = 0;
    virtual void Write32 (unsigned long int address, unsigned long int  value)  = 0;
    virtual void Write16 (unsigned long int address, unsigned short int value)  = 0;

    virtual void Read (unsigned long int address, char * data, size_t count)  = 0;
    virtual unsigned long  Read32 (unsigned long int address)  = 0;
    virtual unsigned short Read16 (unsigned long int address)  = 0;
};




#endif



